/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: test_bitset.c,v $
*	$Revision: 1.7 $
*	$Date: 2007/11/29 10:58:45 $
*	$Author: i.zapreev $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
*	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
*	automated verification of properties concerning long-run and instantaneous rewards
*	as well as cumulative rewards.
*	
*	Copyright (C) The University of Twente, 2004-2006.
*	Authors: Joost-Pieter Katoen, Maneesh Khattri, Ivan Zapreev, Tim Kemna
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Our contacts:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: This is for testing the bitset only.
*	Uses: Definition of bitset - bitset.h
*/

# include "macro.h"
# include "bitset.h"

/*
NOTE: THE TESTS IN THIS FILE SHOULD NOT FAIL. IF ONE OF THE
TESTS FAILS - SOMETHING'S REALLY WRONG.

IN CONTRAST TO THE TESTS HERE IN PRACTICE SOME INVOCATIONS
CAN FAIL DUE TO SPURIOUS INPUTS PROVIDED THAT THE TESTS HERE
WERE SUCCESSFUL. CONSEQUENTLY IT IS NECESSARY TO CHECK FOR
SPURIOUS INPUTS IN PRACTICAL SCENARIOS.
*/

/* this is for testing the bitset only*/
int main()
{
	bitset *a = get_new_bitset(38), *b = get_new_bitset(38), *c, *d;
	BITSET_BLOCK_TYPE get1;
	/* test set/get */
	printf("Testing set/get\n");
	set_bit_val(a,31, BIT_ON);
	get1 = get_bit_val(a,31 );
	printf("a.val[31]=%d\n", (get1 ? 1 : 0 ) );
	set_bit_val(a,34, BIT_ON);
	get1 = get_bit_val(a,34 );
	printf("a.val[34]=%d\n", (get1 ? 1 : 0 ) );
	set_bit_val(a,35, BIT_ON);
	get1 = get_bit_val(a,35);
	printf("a.val[35]=%d\n",(get1 ? 1 : 0 ) );
	print_bitset_states(a);
	printf("\ncount a: %d\n", count_non_zero(a));
	set_bit_val(b,0, BIT_ON);
	get1 = get_bit_val(b,0);
	printf("b.val[0]=%d\n", (get1 ? 1 : 0 ) );
	set_bit_val(b,31, BIT_ON);
	get1 = get_bit_val(b,31);
	printf("b.val[31]=%d\n", (get1 ? 1 : 0 ) );
	set_bit_val(b,37, BIT_ON);
	get1 = get_bit_val(b,37);
	printf("b.val[37]=%d\n", (get1 ? 1 : 0 ) );
	print_bitset_states(b);
	printf("\ncount b: %d\n", count_non_zero(b));
	
	/*test or*/
	printf("Testing Or\n");
	c = or(a,b);
	get1 = get_bit_val(c,0);
	printf("c.val[0]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,31);
	printf("c.val[31]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,37);
	printf("c.val[37]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,1);
	printf("c.val[1]=%d\n", (get1 ? 1 : 0 ) );
	print_bitset_states(c);
	printf("\ncount c: %d\n", count_non_zero(c));

	/*test and*/
	printf("Testing And\n");
	c = and(a,b);
	get1 = get_bit_val(c,0);
	printf("c.val[0]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,31);
	printf("c.val[31]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,37);
	printf("c.val[37]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(c,1);
	printf("c.val[1]=%d\n", (get1 ? 1 : 0 ) );
	print_bitset_states(c);
	printf("\ncount c: %d\n", count_non_zero(c));

	/*test not*/
	printf("Testing Not\n");
	d = not(c);
	get1 = get_bit_val(d,0);
	printf("d.val[0]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(d,31);
	printf("d.val[31]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(d,37);
	printf("d.val[37]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(d,38);
	printf("d.val[38]=%d\n", (get1 ? 1 : 0 ) );
	get1 = get_bit_val(d,1);
	printf("d.val[1]=%d\n", (get1 ? 1 : 0 ) );
	print_bitset_states(d);
	printf("\ncount d: %d\n", count_non_zero(d));
	
	return 0;
}
